@@ -5,7 +5,6 @@
 package main
 
 import (
-	"context"
 	"flag"
 	"log"
 	"net/http"
@@ -21,7 +20,6 @@ var (
 
 func main() {
 	flag.Parse()
-	ctx := context.Background()
 
 	s := new(tsnet.Server)
 	defer s.Close()
@@ -32,7 +30,7 @@ func main() {
 	}
 
 	// Serve the Tailscale web client.
-	ws, cleanup := web.NewServer(ctx, web.ServerOpts{
+	ws, cleanup := web.NewServer(web.ServerOpts{
 		DevMode:     *devMode,
 		LocalClient: lc,
 	})
