@@ -101,7 +101,8 @@ func (nl *Logger) Startup(nodeID tailcfg.StableNodeID, nodeLogID, domainLogID lo
 	}
 
 	// Startup a log stream to Tailscale's logging service.
-	httpc := &http.Client{Transport: logpolicy.NewLogtailTransport(logtail.DefaultHost, netMon)}
+	logf := log.Printf
+	httpc := &http.Client{Transport: logpolicy.NewLogtailTransport(logtail.DefaultHost, netMon, logf)}
 	if testClient != nil {
 		httpc = testClient
 	}
@@ -123,7 +124,7 @@ func (nl *Logger) Startup(nodeID tailcfg.StableNodeID, nodeLogID, domainLogID lo
 		// Include process sequence numbers to identify missing samples.
 		IncludeProcID:       true,
 		IncludeProcSequence: true,
-	}, log.Printf)
+	}, logf)
 	nl.logger.SetSockstatsLabel(sockstats.LabelNetlogLogger)
 
 	// Startup a data structure to track per-connection statistics.
